<?php

/**
 * @file
 * Contains Drupal\library\Plugin\Field\FieldWidget\LibraryItemFieldWidget.
 */

namespace Drupal\library\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\library\Entity\LibraryItem;

/**
 * Plugin implementation of the 'library_item_field_widget' widget.
 *
 * @FieldWidget(
 *   id = "library_item_field_widget",
 *   label = @Translation("Library item widget"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class LibraryItemFieldWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $items = $items->referencedEntities();
    /* @var LibraryItem $items[] */

    $element['library'] = array(
      '#type' => 'fieldset',
      '#title' => t('Item'),
      '#attributes' => array('class' => array('container-inline')),
    );

    if (isset($items[$delta])) {
      $element['library']['item_id'] = array(
        '#type' => 'hidden',
        '#value' => $items[$delta]->id(),
      );
    }


    $element['library']['barcode'] = array(
      '#type' => 'textfield',
      '#default_value' => 0,
      '#size' => 22,
      '#default_value' => isset($items[$delta]) ? $items[$delta]->get('barcode')->value : '',
      '#maxlength' => $this->getFieldSetting('max_length'),
      '#title' => t('Barcode'),
    );
    if ($this->generateBarcode()) {
      $element['library']['barcode']['#disabled'] = 'true';
      $element['library']['barcode']['#placeholder'] = t('Automatically generated');

    }
    $element['library']['in_circulation'] = array(
      '#type' => 'checkbox',
      '#default_value' => isset($items[$delta]) ? $items[$delta]->get('in_circulation')->value : 0,
      '#title' => t('Reference only'),
    );
    $element['library']['library_status'] = array(
      '#type' => 'hidden',
      '#default_value' =>  isset($items[$delta]) ? $items[$delta]->get('library_status')->value : 0,
    );
    $element['library']['notes'] = array(
        '#type' => 'textfield',
        '#default_value' => isset($items[$delta]) ? $items[$delta]->get('notes')->value : NULL,
        '#size' => '22',
        '#maxlength' => $this->getFieldSetting('max_length'),
        '#title' => t('Notes'),
    );

    if (isset($items[$delta])) {
      $element['library']['remove'] = array(
        '#type' => 'checkbox',
        '#default_value' => 0,
        '#title' => t('Remove'),
      );
    } else {
      $element['library']['create'] = array(
        '#type' => 'checkbox',
        '#default_value' => 0,
        '#title' => t('Create'),
      );
    }



    return $element;
  }

  protected function generateBarcode() {
    // TODO: FORM to set this.
    $entity = $this->fieldDefinition->getTargetEntityTypeId();
    $field = $this->fieldDefinition->getName();
    $limit = \Drupal::state()->get('library.barcode_base.' . $entity . '.' . $field);
    if ($limit) {
      return true;
    } else {
      return false;
    }
  }

  public static function findBarcode() {
    $query = \Drupal::EntityQuery('library_item')
      ->sort('barcode', 'DESC')
      ->range(0,1)
      ->execute();
    foreach ($query as $element) {
      $item = entity_load('library_item', $element);
    }
    return $item->get('barcode')->value;

  }

  // TODO: isApplicable()

}
